@@ -4,16 +4,18 @@ import logging
 from timeit import default_timer
 import sys
 
-from dask.core import flatten
 from toolz import valmap
 from tornado import gen
 from tornado.ioloop import IOLoop
 
 from .progress import format_time, Progress, MultiProgress
 
-from ..core import connect, read, write, dumps
+from ..compatibility import html_escape
+from ..core import connect, coerce_to_address, CommClosedError
 from ..client import default_client, futures_of
-from ..utils import sync, ignoring, key_split, is_kernel
+from ..protocol.pickle import dumps
+from ..utils import (ignoring, key_split, is_kernel, LoopRunner,
+        parse_timedelta)
 
 
 logger = logging.getLogger(__name__)
@@ -21,21 +23,16 @@ logger = logging.getLogger(__name__)
 
 def get_scheduler(scheduler):
     if scheduler is None:
-        scheduler = default_client().scheduler
-        return scheduler.ip, scheduler.port
-    if isinstance(scheduler, str):
-        scheduler = scheduler.split(':')
-    if isinstance(scheduler, tuple):
-        return scheduler
-    raise TypeError("Expected None, string, or tuple")
+        return default_client().scheduler.address
+    return coerce_to_address(scheduler)
 
 
 class ProgressBar(object):
-    def __init__(self, keys, scheduler=None, interval=0.1, complete=True):
+    def __init__(self, keys, scheduler=None, interval='100ms', complete=True):
         self.scheduler = get_scheduler(scheduler)
 
         self.keys = {k.key if hasattr(k, 'key') else k for k in keys}
-        self.interval = interval
+        self.interval = parse_timedelta(interval, default='s')
         self.complete = complete
         self._start_time = default_timer()
 
@@ -59,21 +56,24 @@ class ProgressBar(object):
                     'remaining': len(p.keys),
                     'status': p.status}
 
-        self.stream = yield connect(*self.scheduler)
+        self.comm = yield connect(self.scheduler)
         logger.debug("Progressbar Connected to scheduler")
 
-        yield write(self.stream, {'op': 'feed',
-                                  'setup': dumps(setup),
-                                  'function': dumps(function),
-                                  'interval': self.interval})
+        yield self.comm.write({'op': 'feed',
+                               'setup': dumps(setup),
+                               'function': dumps(function),
+                               'interval': self.interval})
 
         while True:
-            response = yield read(self.stream)
+            try:
+                response = yield self.comm.read()
+            except CommClosedError:
+                break
             self._last_response = response
             self.status = response['status']
             self._draw_bar(**response)
             if response['status'] in ('error', 'finished'):
-                self.stream.close()
+                yield self.comm.close()
                 self._draw_stop(**response)
                 break
 
@@ -84,19 +84,20 @@ class ProgressBar(object):
 
     def __del__(self):
         with ignoring(AttributeError):
-            self.stream.close()
+            self.comm.abort()
 
 
 class TextProgressBar(ProgressBar):
-    def __init__(self, keys, scheduler=None, interval=0.1, width=40,
-                       loop=None, complete=True, start=True):
+    def __init__(self, keys, scheduler=None, interval='100ms', width=40,
+                 loop=None, complete=True, start=True):
         super(TextProgressBar, self).__init__(keys, scheduler, interval,
                                               complete)
         self.width = width
         self.loop = loop or IOLoop()
 
         if start:
-            sync(self.loop, self.listen)
+            loop_runner = LoopRunner(self.loop)
+            loop_runner.run_sync(self.listen)
 
     def _draw_bar(self, remaining, all, **kwargs):
         frac = (1 - remaining / all) if all else 1.0
@@ -118,17 +119,18 @@ class ProgressWidget(ProgressBar):
     progress: User function
     TextProgressBar: Text version suitable for the console
     """
-    def __init__(self, keys, scheduler=None, interval=0.1,
-                complete=False, loop=None):
+
+    def __init__(self, keys, scheduler=None, interval='100ms',
+                 complete=False, loop=None):
         super(ProgressWidget, self).__init__(keys, scheduler, interval,
                                              complete)
 
         from ipywidgets import FloatProgress, HBox, VBox, HTML
         self.elapsed_time = HTML('')
-        self.bar = FloatProgress(min=0, max=1, description='', height = '10px')
-        self.bar_text = HTML('', width = "140px")
+        self.bar = FloatProgress(min=0, max=1, description='', height='10px')
+        self.bar_text = HTML('', width="140px")
 
-        self.bar_widget = HBox([ self.bar_text, self.bar ])
+        self.bar_widget = HBox([self.bar_text, self.bar])
         self.widget = VBox([self.elapsed_time, self.bar_widget])
 
     def _ipython_display_(self, **kwargs):
@@ -138,19 +140,23 @@ class ProgressWidget(ProgressBar):
     def _draw_stop(self, remaining, status, **kwargs):
         if status == 'error':
             self.bar.bar_style = 'danger'
-            self.elapsed_time.value = '<div style="padding: 0px 10px 5px 10px"><b>Warning:</b> the computation terminated due to an error after ' + format_time(self.elapsed) + '</div>'
+            self.elapsed_time.value = '<div style="padding: 0px 10px 5px 10px"><b>Exception:</b> ' + \
+                format_time(self.elapsed) + '</div>'
         elif not remaining:
             self.bar.bar_style = 'success'
+            self.elapsed_time.value = '<div style="padding: 0px 10px 5px 10px"><b>Finished:</b> ' + \
+                format_time(self.elapsed) + '</div>'
 
     def _draw_bar(self, remaining, all, **kwargs):
         ndone = all - remaining
-        self.elapsed_time.value = '<div style=\"padding: 0px 10px 5px 10px\"><b>Elapsed time:</b> ' + format_time(self.elapsed) + '</div>'
+        self.elapsed_time.value = '<div style=\"padding: 0px 10px 5px 10px\"><b>Computing:</b> ' + \
+            format_time(self.elapsed) + '</div>'
         self.bar.value = ndone / all if all else 1.0
         self.bar_text.value = '<div style="padding: 0px 10px 0px 10px; text-align:right;">%d / %d</div>' % (ndone, all)
 
 
 class MultiProgressBar(object):
-    def __init__(self, keys, scheduler=None, func=key_split, interval=0.1, complete=False):
+    def __init__(self, keys, scheduler=None, func=key_split, interval='100ms', complete=False):
         self.scheduler = get_scheduler(scheduler)
 
         self.keys = {k.key if hasattr(k, 'key') else k for k in keys}
@@ -180,21 +186,21 @@ class MultiProgressBar(object):
                     'remaining': valmap(len, p.keys),
                     'status': p.status}
 
-        self.stream = yield connect(*self.scheduler)
+        self.comm = yield connect(self.scheduler)
         logger.debug("Progressbar Connected to scheduler")
 
-        yield write(self.stream, {'op': 'feed',
-                                  'setup': dumps(setup),
-                                  'function': dumps(function),
-                                  'interval': self.interval})
+        yield self.comm.write({'op': 'feed',
+                               'setup': dumps(setup),
+                               'function': dumps(function),
+                               'interval': self.interval})
 
         while True:
-            response = yield read(self.stream)
+            response = yield self.comm.read()
             self._last_response = response
             self.status = response['status']
             self._draw_bar(**response)
             if response['status'] in ('error', 'finished'):
-                self.stream.close()
+                yield self.comm.close()
                 self._draw_stop(**response)
                 break
         logger.debug("Progressbar disconnected from scheduler")
@@ -204,7 +210,7 @@ class MultiProgressBar(object):
 
     def __del__(self):
         with ignoring(AttributeError):
-            self.stream.close()
+            self.comm.abort()
 
 
 class MultiProgressWidget(MultiProgressBar):
@@ -219,6 +225,7 @@ class MultiProgressWidget(MultiProgressBar):
     MultiProgress: Non-visualization component that contains most logic
     ProgressWidget: Single progress bar widget
     """
+
     def __init__(self, keys, scheduler=None, minimum=0, interval=0.1, func=key_split,
                  complete=False):
         super(MultiProgressWidget, self).__init__(keys, scheduler, func, interval, complete)
@@ -227,25 +234,30 @@ class MultiProgressWidget(MultiProgressBar):
 
     def make_widget(self, all):
         from ipywidgets import FloatProgress, HBox, VBox, HTML
-        import cgi
         self.elapsed_time = HTML('')
         self.bars = {key: FloatProgress(min=0, max=1, description='',
                                         height='10px')
-                        for key in all}
-        self.bar_texts = {key: HTML('', width = "140px") for key in all}
-        self.bar_labels = {key: HTML('<div style=\"padding: 0px 10px 0px 10px; text-align:left; word-wrap: break-word;\">' + cgi.escape(key.decode() if isinstance(key, bytes) else key) + '</div>')
-                            for key in all}
+                     for key in all}
+        self.bar_texts = {key: HTML('', width="140px") for key in all}
+        self.bar_labels = {key: HTML('<div style=\"padding: 0px 10px 0px 10px;'
+                                     ' text-align:left; word-wrap: '
+                                     'break-word;\">' +
+                                     html_escape(key.decode()
+                                                 if isinstance(key, bytes)
+                                                 else key) +
+                                     '</div>')
+                           for key in all}
 
-        def key(kv):
+        def keyfunc(kv):
             """ Order keys by most numerous, then by string name """
             return kv[::-1]
 
-        key_order = [k for k, v in sorted(all.items(), key=key, reverse=True)]
+        key_order = [k for k, v in sorted(all.items(), key=keyfunc, reverse=True)]
 
-        self.bar_widgets = VBox([ HBox([ self.bar_texts[key],
-                                         self.bars[key],
-                                         self.bar_labels[key] ])
-                                for key in key_order ])
+        self.bar_widgets = VBox([HBox([self.bar_texts[key],
+                                       self.bars[key],
+                                       self.bar_labels[key]])
+                                 for key in key_order])
         self.widget.children = (self.elapsed_time, self.bar_widgets)
 
     def _ipython_display_(self, **kwargs):
@@ -256,21 +268,27 @@ class MultiProgressWidget(MultiProgressBar):
         for k, v in remaining.items():
             if not v:
                 self.bars[k].bar_style = 'success'
+            else:
+                self.bars[k].bar_style = 'danger'
 
-        """ TODO
         if status == 'error':
-            self.bars[self.func(key)].bar_style = 'danger'
-            self.elapsed_time.value = '<div style="padding: 0px 10px 5px 10px"><b>Warning:</b> the computation terminated due to an error after ' + format_time(self.elapsed) + '</div>'
-        """
+            # self.bars[self.func(key)].bar_style = 'danger'  # TODO
+            self.elapsed_time.value = '<div style="padding: 0px 10px 5px 10px"><b>Exception:</b> ' + \
+                format_time(self.elapsed) + '</div>'
+        else:
+            self.elapsed_time.value = '<div style="padding: 0px 10px 5px 10px"><b>Finished:</b> ' + \
+                format_time(self.elapsed) + '</div>'
 
     def _draw_bar(self, remaining, all, status, **kwargs):
         if self.keys and not self.widget.children:
             self.make_widget(all)
         for k, ntasks in all.items():
             ndone = ntasks - remaining[k]
-            self.elapsed_time.value = '<div style="padding: 0px 10px 5px 10px"><b>Elapsed time:</b> ' + format_time(self.elapsed) + '</div>'
+            self.elapsed_time.value = '<div style="padding: 0px 10px 5px 10px"><b>Computing:</b> ' + \
+                format_time(self.elapsed) + '</div>'
             self.bars[k].value = ndone / ntasks if ntasks else 1.0
-            self.bar_texts[k].value = '<div style="padding: 0px 10px 0px 10px; text-align: right">%d / %d</div>' % (ndone, ntasks)
+            self.bar_texts[k].value = '<div style="padding: 0px 10px 0px 10px; text-align: right">%d / %d</div>' % (
+                ndone, ntasks)
 
 
 def progress(*futures, **kwargs):
