@@ -42,10 +42,7 @@ class PublishExtension(object):
 
     def get(self, stream, name=None, client=None):
         with log_errors():
-            if name in self.datasets:
-                return self.datasets[name]
-            else:
-                raise KeyError("Dataset '%s' not found" % name)
+            return self.datasets.get(name, None)
 
 
 class Datasets(MutableMapping):
@@ -68,9 +65,6 @@ class Datasets(MutableMapping):
     def __delitem__(self, key):
         self.__client.unpublish_dataset(key)
 
-    def __contains__(self, key):
-        return key in self.__client.list_datasets()
-
     def __iter__(self):
         for key in self.__client.list_datasets():
             yield key
