@@ -8,9 +8,11 @@ import weakref
 from tornado import gen
 import pytest
 
+import dask
 from distributed.compatibility import finalize
 from distributed.core import (pingpong, Server, rpc, connect, send_recv,
                                coerce_to_address, ConnectionPool)
+from distributed.protocol.compression import compressions
 
 from distributed.metrics import time
 from distributed.protocol import to_serialize
@@ -56,6 +58,10 @@ def echo_serialize(comm, x):
     return {'result': to_serialize(x)}
 
 
+def echo_no_serialize(comm, x):
+    return {'result': x}
+
+
 def test_server(loop):
     """
     Simple Server test.
@@ -572,3 +578,23 @@ def test_tick_logging(s, a, b):
         assert 'Scheduler' in text or 'Worker' in text
     finally:
         core.tick_maximum_delay = old
+
+
+@pytest.mark.parametrize('compression', list(compressions))
+@pytest.mark.parametrize('serialize', [echo_serialize, echo_no_serialize])
+def test_compression(compression, serialize, loop):
+    with dask.set_options(compression=compression):
+
+        @gen.coroutine
+        def f():
+            server = Server({'echo': serialize})
+            server.listen('tcp://')
+
+            with rpc(server.address) as r:
+                data = b'1' * 1000000
+                result = yield r.echo(x=to_serialize(data))
+                assert result == {'result': data}
+
+            server.stop()
+
+        loop.run_sync(f)
