@@ -534,7 +534,8 @@ class WorkerBase(ServerNode):
         try:
             compressed = yield comm.write(msg)
         except EnvironmentError:
-            logger.exception('failed during get data', exc_info=True)
+            logger.exception('failed during get data with %s -> %s',
+                             self.address, who, exc_info=True)
             comm.abort()
             raise
         stop = time()
@@ -1086,6 +1087,7 @@ class Worker(WorkerBase):
         self.profile_history = deque(maxlen=3600)
 
         self.priorities = dict()
+        self.generation = 0
         self.durations = dict()
         self.startstops = defaultdict(list)
         self.resource_restrictions = dict()
@@ -1245,6 +1247,10 @@ class Worker(WorkerBase):
                 if state in IN_PLAY:
                     return
 
+            if priority is not None:
+                priority = tuple(priority) + (self.generation,)
+                self.generation -= 1
+
             if self.dep_state.get(key) == 'memory':
                 self.task_state[key] = 'memory'
                 self.send_task_state_to_scheduler(key)
